@@ -1,10 +1,11 @@
 extern crate chrono;
-extern crate glob;
 
 use self::chrono::NaiveDateTime;
 use num_derive::{FromPrimitive, ToPrimitive};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
+use std::path::Path;
+use std::path::PathBuf;
 
 use super::commit;
 use super::event;
@@ -33,34 +34,47 @@ pub struct ItemList {
     /// List of events
     pub events: Vec<event::Event>,
     /// Base path that was used to create the item list
-    pub path: String,
+    pub path: PathBuf,
 }
 
 impl ItemList {
-    /// Synchronize an existing items list with the items found in a path
-    pub fn synchronize(&mut self, path: &str) {
-        let mut found_item_paths = find_items(path);
+    /// Remove all missing files from the item list
+    pub fn drain_missing(&mut self) {
+        self.items = self.items.drain(..).filter(|i| i.path.exists()).collect();
+    }
 
-        self.items = self
-            .items
-            .drain(..)
-            .filter(|i| i.get_path().exists() && found_item_paths.contains(i.get_path_as_str()))
-            .collect();
-
-        // Add all newly found
-        for item_path in found_item_paths.drain(..) {
-            let item = Self::create_item(item_path, true, "");
-            let path = item.get_path();
-            if !self.items.iter().any(|i| i.get_path() == path) {
-                self.items.push(item);
+    /// Check if a path can be added
+    pub fn check_and_add(&mut self, path: &Path) -> bool {
+        if let Some(extension) = path.extension() {
+            if let Some(extension) = extension.to_ascii_lowercase().to_str() {
+                if file_item::FileItem::get_extensions().contains(&extension)
+                    && !self.items.iter().any(|i| i.path == path)
+                {
+                    let item = Self::create_item(path.to_path_buf(), true, "");
+                    self.items.push(item);
+                }
             }
         }
+        false
+    }
+
+    /// Finish the synchronization progress
+    pub fn finish_synchronizing(&mut self, base_path: &Path) {
         self.items.sort();
-        self.path = String::from(path);
+        self.path = base_path.to_path_buf();
+    }
+
+    /// Adds an item to the list
+    pub fn add_item(&mut self, item_path: &Path, take_over: bool, encoded_hash: &str) {
+        self.items.push(Self::create_item(
+            item_path.to_path_buf(),
+            take_over,
+            encoded_hash,
+        ));
     }
 
     /// Internal function to create a new file item
-    fn create_item(item_path: String, take_over: bool, encoded_hash: &str) -> file_item::FileItem {
+    fn create_item(item_path: PathBuf, take_over: bool, encoded_hash: &str) -> file_item::FileItem {
         let resolver = resolvers::get_resolver(&item_path);
         file_item::FileItem::new(item_path, resolver, take_over, encoded_hash)
     }
@@ -81,13 +95,10 @@ impl ItemList {
                 if (index == self.items.len())
                     || (timestamp + max_diff_seconds < self.items[index].get_timestamp())
                 {
+                    let similars = start_similar_index..index;
                     // The item has a larger diff, so set all items between start_similar_index and index to be similar to each other
                     for similar_index in start_similar_index..index {
-                        for other_index in start_similar_index..index {
-                            if similar_index != other_index {
-                                self.items[similar_index].add_similar(other_index);
-                            }
-                        }
+                        self.items[similar_index].add_similars(&similars);
                     }
                     start_similar_index = index;
                 }
@@ -96,6 +107,9 @@ impl ItemList {
                 }
             }
         }
+        for index in 0..self.items.len() {
+            self.items[index].clean_similars(index);
+        }
     }
 
     /// Go through all images and find similar ones by comparing the hash
@@ -127,6 +141,7 @@ impl ItemList {
             for (_, other_index) in similar_list {
                 self.items[index].add_similar(*other_index);
             }
+            self.items[index].clean_similars(index);
         }
     }
 
@@ -134,7 +149,7 @@ impl ItemList {
     /// The progress is reported by calling a callback function with the file that is currently processed.
     pub fn commit(
         &self,
-        path: &str,
+        path: &Path,
         commit_method: CommitMethod,
         progress_callback: impl Fn(String),
     ) {
@@ -154,26 +169,6 @@ impl ItemList {
     }
 }
 
-/// Find all files in a directory with the extensions supported by FileItem
-fn find_items(path: &str) -> Vec<String> {
-    let match_options = glob::MatchOptions {
-        case_sensitive: false,
-        require_literal_leading_dot: false,
-        require_literal_separator: false,
-    };
-    let mut files: Vec<String> = Vec::new();
-
-    for extension in file_item::FileItem::get_extensions() {
-        let glob_pattern = format!("{}/**/*.{}", path, extension);
-        let entries = glob::glob_with(&glob_pattern, match_options).unwrap();
-        for entry in entries {
-            let file = entry.expect("IO error");
-            files.push(file.to_str().unwrap().to_string());
-        }
-    }
-    files
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -203,7 +198,7 @@ mod tests {
         let mut items: Vec<file_item::FileItem> = vec![];
         for _ in 0..6 {
             items.push(file_item::FileItem::new(
-                String::from(""),
+                PathBuf::from(""),
                 Box::new(MockResolver {}),
                 true,
                 "",
@@ -212,7 +207,7 @@ mod tests {
         let mut item_list = ItemList {
             items,
             events: vec![],
-            path: String::from(""),
+            path: PathBuf::from(""),
         };
 
         item_list.find_similar(5);
@@ -224,4 +219,28 @@ mod tests {
         assert_eq!(1, item_list.items[4].get_similars().len());
         assert_eq!(1, item_list.items[5].get_similars().len());
     }
+
+    #[test]
+    fn updating() {
+        let mut item_list = ItemList {
+            items: vec![],
+            events: vec![],
+            path: PathBuf::from(""),
+        };
+
+        item_list.check_and_add(Path::new("tests/test_no_date.jpg"));
+        item_list.check_and_add(Path::new("tests/test_no_exif.jpg"));
+        item_list.check_and_add(Path::new("tests/test.jpg"));
+        item_list.check_and_add(Path::new("tests/test_no_date.jpg"));
+        item_list.check_and_add(Path::new("tests/test"));
+        assert_eq!(3, item_list.items.len());
+
+        item_list.finish_synchronizing(Path::new("tests"));
+        assert_eq!("tests", item_list.path.to_str().unwrap());
+
+        item_list.add_item(Path::new("tests/not_there"), true, "");
+        assert_eq!(4, item_list.items.len());
+        item_list.drain_missing();
+        assert_eq!(3, item_list.items.len());
+    }
 }
