@@ -10,6 +10,7 @@ use rfd::FileDialog;
 use sixtyfps::{Model, SharedString};
 use std::collections::HashMap;
 use std::fmt::Debug;
+use std::path::{Path, PathBuf};
 use std::rc::Rc;
 use std::sync::Mutex;
 use std::thread;
@@ -48,6 +49,7 @@ pub struct MainWindow {
     events_model: Rc<sixtyfps::VecModel<Event>>,
     commit_result_model: Rc<sixtyfps::VecModel<CommitResult>>,
     image_cache: Rc<ImageCache>,
+    synchronizer: Rc<Synchronizer>,
 }
 
 impl Default for MainWindow {
@@ -75,7 +77,7 @@ impl MainWindow {
         let item_list = ItemList {
             items: vec![],
             events: vec![],
-            path: String::new(),
+            path: PathBuf::new(),
         };
 
         let item_list = Arc::new(Mutex::new(item_list));
@@ -86,10 +88,15 @@ impl MainWindow {
 
         // Construct main window
         let image_sieve = ImageSieve::new();
-        let synchronizer = Synchronizer::new(item_list.clone(), &image_sieve);
 
-        // Start synchronization in a background thread
-        synchronizer.synchronize(&settings.source_directory, settings.clone());
+        let synchronizer = Synchronizer::new(item_list.clone(), &image_sieve);
+        if !settings.source_directory.is_empty() {
+            // Start synchronization in a background thread
+            synchronizer.synchronize(
+                Some(Path::new(&settings.source_directory)),
+                settings.clone(),
+            );
+        }
         let mut cache = ImageCache::new();
         cache.restrict_size(1600, 1000);
 
@@ -102,6 +109,7 @@ impl MainWindow {
             events_model: event_list_model,
             commit_result_model,
             image_cache: Rc::new(cache),
+            synchronizer: Rc::new(synchronizer),
         };
 
         // Set initial values
@@ -110,6 +118,10 @@ impl MainWindow {
             .window
             .set_window_title(SharedString::from("ImageSieve v") + version);
         settings.to_window(&main_window.window);
+        if settings.source_directory.is_empty() {
+            main_window.window.set_loading(false);
+            main_window.window.set_calculating_similarities(false);
+        }
 
         // Set model references
         main_window
@@ -140,16 +152,17 @@ impl MainWindow {
     pub fn run(&self) {
         self.window.run();
 
+        self.synchronizer.stop();
+
         // Save settings when program exits
         let settings = Settings::from_window(&self.window);
         JsonPersistence::save(&get_settings_filename(), &settings);
 
         // and save item list
         let item_list = self.item_list.lock().unwrap();
-        JsonPersistence::save(
-            &get_project_filename(item_list.path.as_str()),
-            &item_list.clone(),
-        );
+        if !item_list.items.is_empty() || !item_list.events.is_empty() {
+            JsonPersistence::save(&get_project_filename(&item_list.path), &item_list.clone());
+        }
     }
 
     /// Setup sixtyfps GUI callbacks
@@ -220,7 +233,7 @@ impl MainWindow {
             let events_model = self.events_model.clone();
             let item_list = self.item_list.clone();
             let window_weak = self.window.as_weak();
-            let synchronizer = Synchronizer::new(self.item_list.clone(), &self.window);
+            let synchronizer = self.synchronizer.clone();
 
             move || {
                 let file_dialog = FileDialog::new();
@@ -230,24 +243,23 @@ impl MainWindow {
                         let item_list = item_list.lock().unwrap();
                         if !item_list.items.is_empty() {
                             JsonPersistence::save(
-                                &get_project_filename(item_list.path.as_str()),
+                                &get_project_filename(&item_list.path),
                                 &item_list.clone(),
                             );
                         }
                     }
 
-                    let source_path = folder.to_str().unwrap();
                     empty_model(item_list_model.clone());
                     empty_model(events_model.clone());
 
                     // Synchronize in a background thread
                     window_weak.unwrap().set_loading(true);
                     synchronizer
-                        .synchronize(source_path, Settings::from_window(&window_weak.unwrap()));
+                        .synchronize(Some(&folder), Settings::from_window(&window_weak.unwrap()));
 
                     window_weak
                         .unwrap()
-                        .set_source_directory(SharedString::from(source_path));
+                        .set_source_directory(SharedString::from(folder.to_str().unwrap()));
                 }
             }
         });
@@ -312,13 +324,10 @@ impl MainWindow {
                     end_date.as_str(),
                 )
                 .unwrap();
-                events_model.push(Event {
-                    name,
-                    start_date,
-                    end_date,
-                });
                 let mut item_list = item_list.lock().unwrap();
                 item_list.events.push(event);
+                item_list.events.sort_unstable();
+                synchronize_event_list_model(&item_list, &events_model);
                 // Synchronize the item list to update the icons of the entries
                 synchronize_item_list_model(&item_list, &item_list_model.clone());
             }
@@ -342,6 +351,8 @@ impl MainWindow {
                     event.start_date.as_str(),
                     event.end_date.as_str(),
                 ) {
+                    item_list.events.sort_unstable();
+                    synchronize_event_list_model(&item_list, &events_model);
                     synchronize_item_list_model(&item_list, &item_list_model.clone());
                 }
             }
@@ -366,19 +377,27 @@ impl MainWindow {
             let item_list = self.item_list.clone();
             move |i: i32| {
                 let item_list = item_list.lock().unwrap();
-                opener::open(item_list.items[i as usize].get_path()).ok();
+                let item = &item_list.items[i as usize];
+                opener::open(&item.path).ok();
             }
         });
 
         self.window.on_recheck_similarities({
             // Browse source was clicked, select new path
             let window_weak = self.window.as_weak();
-            let synchronizer = Synchronizer::new(self.item_list.clone(), &self.window);
+            let synchronizer = self.synchronizer.clone();
 
             move || {
                 // Synchronize in a background thread
                 window_weak.unwrap().set_calculating_similarities(true);
-                synchronizer.synchronize("", Settings::from_window(&window_weak.unwrap()));
+                synchronizer.synchronize(None, Settings::from_window(&window_weak.unwrap()));
+            }
+        });
+
+        self.window.on_cancel_loading({
+            let synchronizer = self.synchronizer.clone();
+            move || {
+                synchronizer.stop();
             }
         });
     }
@@ -414,7 +433,7 @@ pub fn synchronize_event_list_model(
     item_list: &ItemList,
     event_list_model: &sixtyfps::VecModel<Event>,
 ) {
-    let empty_model = event_list_model.row_count() == 0;
+    let model_count = event_list_model.row_count();
     // Event model
     for (index, event) in item_list.events.iter().enumerate() {
         let _event = Event {
@@ -422,7 +441,7 @@ pub fn synchronize_event_list_model(
             start_date: SharedString::from(event.start_date_as_string()),
             end_date: SharedString::from(event.end_date_as_string()),
         };
-        if empty_model {
+        if index >= model_count {
             event_list_model.push(_event);
         } else {
             event_list_model.set_row_data(index, _event);
@@ -587,6 +606,6 @@ pub fn commit(
                 commit_result_model.push(commit_result);
             });
         };
-        item_list_copy.commit(&target_path, commit_method, progress_callback);
+        item_list_copy.commit(Path::new(&target_path), commit_method, progress_callback);
     });
 }
